/* ===== *
 * Reset *
 * ===== */

html, body, div, span, applet, object, iframe,
h1, h2, h3, h4, h5, h6, p, blockquote, pre,
a, abbr, acronym, address, big, cite, code,
del, dfn, em, img, ins, kbd, q, s, samp,
small, strike, strong, sub, sup, tt, var,
b, u, i, center,
dl, dt, dd, ol, ul, li,
fieldset, form, label, legend,
table, caption, tbody, tfoot, thead, tr, th, td,
article, aside, canvas, details, embed,
figure, figcaption, footer, header, hgroup,
menu, nav, output, ruby, section, summary,
time, mark, audio, video {
    margin: 0;
    padding: 0;
    border: 0;
    font: inherit;
    vertical-align: baseline;
}

/* HTML5 display-role reset for older browsers */
article, aside, details, figcaption, figure,
footer, header, hgroup, menu, nav, section {
    display: block;
}

ol, ul {
    list-style: none;
}

blockquote, q {
}

table {
    border-collapse: collapse;
    border-spacing: 0;
}

a:focus {
    outline: none;
}

/* ============ *
 * Theme Styles *
 * ============ */

body {
    box-sizing: border-box;
    color:#000;
    font-size: 16px;
    font-family: 'yorktenslabnormregular', serif;
    line-height: 1.5;
    -webkit-font-smoothing: antialiased;

    background: #f2f2f2;
    border-top: 0px solid #111;
    border-bottom: 1px solid #111;

    position: relative;
    max-width: 640px;
    padding: 10px 10px;
    margin: 0px auto;
}

h1, h2, h3, h4, h5, h6 {
    margin: 10px 0;
    font-weight: 600;
    color:#000;
    letter-spacing: -0.5px;
}

h1 {
    font-size: 36px;
    font-weight: 600;
}

h2 {
    padding-bottom: 5px;
    font-size: 32px;
    background: url('../img/bg-hr.png') repeat-x bottom;
}

h2 code {
    font-size: 24px;
}

h3 {
    font-size: 24px;
}

h4 {
    font-size: 21px;
}

h5 {
    font-size: 18px;
}

h6 {
    font-size: 16px;
}

p {
    margin: 10px 0 15px 0;
}

footer p {
    color: #f2f2f2;
}

a {
    text-decoration: none;
    color: #007ee0;
    text-shadow: none;
}

a:visited {
    color: #802acb;
}

footer a:hover {
    color: #f2f2f2;
    background-color: #007ee0;
}

em {
    font-style: italic;
}

strong {
    font-weight: bold;
}

img {
    position: relative;
    margin: 0 auto;
    max-width: 739px;
    padding: 5px;
    margin: 10px 0 10px 0;
}

pre, code {
    width: 100%;
    color: #111;
    background-color: #fff;

    font-family: Monaco, "Bitstream Vera Sans Mono", "Lucida Console", Terminal, monospace;
    font-size: 14px;

    border-radius: 2px;
    -moz-border-radius: 2px;
    -webkit-border-radius: 2px;
}

pre {
    width: 100%;
    padding: 4px;
    box-shadow: 0 0 4px rgba(0,0,0,.1);
    overflow: auto;
}

code {
    padding: 3px;
    margin: 0 3px;
    white-space: nowrap;
    box-shadow: 0 0 10px rgba(0,0,0,.1);
}

pre code {
    display: block;
    box-shadow: none;
}

blockquote {
    color: #666;
    margin-bottom: 20px;
    padding: 0 0 0 20px;
    border-left: 3px solid #bbb;
}

ul, ol, dl {
    margin-bottom: 15px
}

ul li {
    list-style: outside;
    padding-left: 0em;
    margin-left: 1em;
}

ol li {
    list-style: decimal inside;
    padding-left: 0em;
    margin-left: 1em;
}

dl dt {
    font-weight: bold;
}

dl dd {
    padding-left: 20px;
}

dl p {
    padding-left: 20px;
}

hr {
    height: 1px;
    margin-bottom: 5px;
    border: none;
    background: url('../images/bg-hr.png') repeat-x center;
}

table {
    border: 1px solid #383838;
    margin-bottom: 20px;
    text-align: left;
}

th {
    font-family: 'Lucida Grande', 'Helvetica Neue', Helvetica, Arial, sans-serif;
    padding: 4px;
    background: #383838;
    color: #fff;
}

td {
    padding: 4px;
    border: 1px solid #383838;
}

form {
    background: #f2f2f2;
}

img {
    /*  width: 100%; */
    max-width: 100%;
}

figure {
    width: 32%;
    display: inline-block;
    text-align: center;
    margin: 10px 0px 0px 0px;
    clear: text;
    padding: 0;
    vertical-align: top;
}

figure img {
    margin: 0;
    padding: 0;
}

figcaption {
    font-style: italic;
    margin: 0;
    padding: 0;
}

/* ===== *
 * Fonts *
 * ===== */

/*!
 * Web Fonts from Fontspring.com
 *
 * All OpenType features and all extended glyphs have been removed.
 * Fully installable fonts can be purchased at http://www.fontspring.com
 *
 * The fonts included in this stylesheet are subject to the End User License you purchased
 * from Fontspring. The fonts are protected under domestic and international trademark and 
 * copyright law. You are prohibited from modifying, reverse engineering, duplicating, or
 * distributing this font software.
 *
 * (c) 2010-2019 Fontspring
 *
 * The fonts included are copyrighted by the vendor listed below.
 *
 * Vendor:      Insigne Design
 * License URL: https://www.fontspring.com/licenses/insigne/webfont
 */

@font-face {
    font-display: swap;
    font-family: 'yorktenslabnormregular';
    src: url('../yorktenslabnormregular-webfont.woff2') format('woff2'),
         url('../yorktenslabnormregular-webfont.woff') format('woff');
    font-weight: normal;
    font-style: normal;
}

/* =================== *
 * Small Device Styles *
 * =================== */

@media screen and (max-width: 500px) {
    body {
        font-size: 16px;
        min-width: 320px;
        max-width: 480px;
    }

    h1 {
        font-size: 28px;
    }

    h2 {
        font-size: 24px;
    }

    h3 {
        font-size: 21px;
    }

    h4 {
        font-size: 18px;
    }

    h5 {
        font-size: 14px;
    }

    h6 {
        font-size: 12px;
    }

    code, pre {
        min-width: 320px;
        max-width: 480px;
        font-size: 11px;
    }

    img {
        width: 100%;
        max-width: 100%;
    }

    figure {
        width: 100%;
        max-width: 100%;
        padding: 5px;
    }

    figure.two {
        width: 100%;
        max-width: 100%;
        padding: 5px;
    }
}
